'use strict';

///////////////////////////////////////////////////////////////////////////////
// @nordoc[package] norris-fs
//
// @desc    A file utility containing common methods needed to manipulate the
//          file system
///////////////////////////////////////////////////////////////////////////////
module.exports.make = function make(deps) {
    if (undefined === deps) deps = {};
    
    // Hoisted variables
    var fs, path, file;

    // Dependency injection
    fs      = deps.fs   || require('fs');
    path    = deps.path || require('path');

    //-------------------------------------------------------------------------
    // @nordoc[module] file
    //
    // @desc    The file module instance
    //-------------------------------------------------------------------------
    file = {

        //---------------------------------------------------------------------
        // @nordoc[method]
        //
        // @desc    Recursively walks a directory, returning all files in a
        //          flat list. This will only recurse for JS files.
        //
        // @args    basepath [string]
        //          The base path to walk from. Note that this has to be an
        //          absolute path (non-absolute paths will cause the method to
        //          throw an exception).
        //
        //          cb [function] (err, files)
        //          The callback is called when the walker has finished walking
        //          all the dirs and sub-dirs.
        //          * err   - the error thrown if the operation failed
        //          * files - the files loaded
        //---------------------------------------------------------------------
        walk : function walk(basepath, cb) {
            // Hoisted variables
            var results, dirs, _walk;

            // Initialize results
            results = [];
            dirs    = [];

            // Make sure there is a callback
            if (undefined === cb)
                cb = function() {}

            // Make sure `base` is an abs path
            if ('/' !== basepath.charAt(0))
                throw new Error('The "base" argument must be an absolute path');

            // Remove trailing slash if exists
            if ('/' === basepath.charAt(basepath.length - 1))
                basepath = basepath.slice(0, -1);

            // Inner function to keep the code base more readable
            _walk = function _walk(base) {
                dirs[base] = base;

                fs.readdir(base, function onReaddir(err, files) {
                    var i, file, stat, newBase;

                    if (err) {
                        cb(err); return;
                    }

                    for (i in files) {
                        file    = files[i];
                        newBase = base + '/' + file;
                        stat    = fs.statSync(newBase);

                        // Reject directories with a dot in the beginning
                        if (stat.isDirectory() && '.' !== file.charAt(0)) {
                            _walk(newBase);
                        } else if (stat.isFile() && '.js' === path.extname(file).toLowerCase()) {
                            results.push(newBase.replace(basepath, '').slice(1));
                        }
                    }

                    process.nextTick(function() {
                        delete dirs[base];

                        if (Object.keys(dirs).length === 0) {
                            cb(null, results);
                        }
                    });
                });
            }

            // Fire off _walk
            _walk(basepath);

            return this;
        }
    }

    return file;
}